/*
   This file is a "library" of functions that work on arrays.
*/

/*
   This method returns the index of the first occurrence of the
   value of target in the array called inputArray. If the value
   of target never occurs in inputArray, then the value returned
   is equal to size (which is not a valid index).
*/
int searchFor(int target, const int inputArray[], int size);



/*
   This method returns the maximum value of the contents
   of the array named inputArray.
*/
int maxInArray(const int inputArray[], int size);



/*
   This method returns the minimum value of the contents
   of the array named inputArray.
*/
int minInArray(const int inputArray[], int size);



/*
   This method increments every entry in the input array.
   Notice that this method changes the array that it is
   passed a reference to. Notice also that this method
   does not have a return value. Its "return values" are
   in the array that it is passed a reference to.
*/
void incArrayInPlace(int inputArray[], int size);



/*
   This method increments every entry in an input array but
   it puts the incremented values in an output array using
   what is sometimes referred to as an "output parameter".
*/
void incArray(const int inputArray[], int size, int outputArray[]);



/*
   This method counts and returns the number of times the
   value of target occurs in the input array.
*/
int numberOfOccurances(int target, const int inputArray[], int size);



/*
   This method returns an array that contains the index values of
   the places in the input array where the value target appears.

   The size of the output array is assumed to be the same as the input array.
   The output array becomes a "partially filled" array. We may not need to
   use all of its entries (why?). The output variable "counter" returns a
   value that indicates how much of the output array has been used (i.e.,
   how much of the "partially filled" array has been filled in).
*/
void allOccurances(int target, const int inputArray[], int size, int outputArray[], int& counter);



/*
   This method sorts an array but it returns the sorted array
   in an output array instead of re-ordering the items in
   the array named inputArray.

   The outputArray is assumed to be the same size as the inputArray.

   This method uses the sortInPlace() method (see below) as
   a helper method.
*/
void sort(const int inputArray[], int size, int outputArray[]);



/*
   This method uses the "bubble sort" algorithm to sort the input array.
   (See pages 531-535 of the textbook for the "selection sort" algorithm.)
*/
void sortInPlace(int inputArray[], int size);



/*
   A helper method for sorting methods.

   Precondition: i and j are legal indices for the array a.
   Postcondition: Values of a[i] and a[j] have been interchanged.
*/
void interchange(int i, int j, int a[]);

