/*
   This is an example of a class that contains an array.

   This class encapsulates the idea of an "infinitely long" array. Of
   course, an array cannot be inifinitely long, so this object creates
   the illusion of infinite length by making the object look like a
   periodic array, that is, an array whose entries keep repeating
   every 10 indices.

   This class is an example of what is called an "abstract data type". This
   class appears to be defining a new kind of data type, the type of an
   inifinitely long (periodic) array.

   Notice that in this version of the class, all of the implementations are
   right here in the header file (kind of like in Java). This is sometimes
   done with classes that have very simple implementations.
*/

class InfiniteArray2
{
   public:
   InfiniteArray2()
   {
      period = 10;
   }

   /*
      Put a value in the InfiniteArray object at "location" i.
   */
   void setEntry(int i, int value)
   {
      int j = i % period;
      hiddenArray[j] = value;
   }

   /*
      Put a value in the InfiniteArray object at "location" i.
   */
   void setEntry(long i, int value)
   {
      int j = (int)(i % (long)period);
      hiddenArray[j] = value;
   }

   /*
      Get the value stored at "location" i.
   */
   int getEntry(int i)
   {
      int j = i % period;
      return hiddenArray[j];
   }

   /*
      Get the value stored at "location" i.
   */
   int getEntry(long i)
   {
      int j = (int)(i % (long)period);
      return hiddenArray[j];
   }

   /*
      Return the period of the infinitly long array.
   */
   int getPeriod()
   {
      return period;
   }

   private:
   // An instance reference variable.
   int period;
   int hiddenArray[10];

};//InfiniteArray2
